@@ -1,7 +1,5 @@
 #!/usr/bin/env python3
 import argparse
-import asyncio
-from aiohttp import request
 from formsite_util.core import FormsiteParams, FormsiteCredentials, FormsiteInterface
 from time import perf_counter
 
@@ -25,7 +23,7 @@ def GatherArguments():
                     "| 5xx  | Unexpected internal error.                  |\n",
                     formatter_class=argparse.RawTextHelpFormatter
     )
-    parser.add_argument('-V', '--version', action='version', version="1.2.6.9")
+    parser.add_argument('-V', '--version', action='version', version="1.2.7.4")
     g_login = parser.add_argument_group('Login')
     g_params = parser.add_argument_group('Results Parameters')
     g_functions = parser.add_argument_group('Functions')
